#ifndef ARDUINO
#include <rxduino.h>
#endif

#ifdef RXDUINO_API
#define INT0N     2
#define INT1N     3
#define PIN_INT0  PIN_P12
#define PIN_INT1  PIN_P13
#else
#define INT0N     0
#define INT1N     1
#define PIN_INT0  2
#define PIN_INT1  3
#endif
#define PIN_DOUT  4
#define PIN_AOUT  5
#define PIN_DIN   6
#define PIN_SOUT1 7
#define PIN_SOUT2 8
#define PIN_SCLK  9
#define PIN_PIN   10
#define PIN_TONE  11
#define PIN_DIN2  12

/* Data Types */
void          *ptr;
boolean       blFlagT;
boolean       blFlagF;
char          cData;
char          *strBuf = "strBuf";
unsigned char ucData;
unsigned char ucStr[10];
byte          bBuf;
byte          byteStr[10];
int           iData;
int           iArray[10];
unsigned int  uiData;
unsigned int  uiArray[10];
word          wData;
word          wArray[10];
long          lData;
long          lArray[10];
unsigned long ulData;
unsigned long ulArray[10];
float         fData;
float         fArray[10];
double        dData;
double        dArray[10];

#ifdef RXDUINO_API
//String not support
#else
String stringStr = "String";
#endif
const float pi = 3.141592;
int sameasglobal = 10;

//for static variable check function
int staticfunction()
{
    static volatile int counter=0;
    return(++counter);
}

//for variable scope check function
void globalcheck()
{
  Serial.print("in globalcheck() sameasglobal=");
  Serial.println(sameasglobal);
}
    
//for interrupt function
volatile int state = LOW;
void blink()
{
  state = !state;
}

void setup()
{
  Serial.begin(9600);
  while(Serial.available() == 0);
  Serial.read();

/* Constants test */
  Serial.println("# Variables");
  Serial.println("## Constants");
  pinMode(PIN_DOUT, OUTPUT);
  pinMode(PIN_DIN, INPUT);
  pinMode(PIN_DIN2, INPUT);
  digitalWrite(PIN_DOUT, HIGH);
  digitalWrite(PIN_DOUT, LOW);
  Serial.print("HIGH=");  Serial.println(HIGH, DEC);
  Serial.print("LOW =");  Serial.println(LOW, DEC);
  Serial.print("INPUT =");  Serial.println(INPUT, DEC);
  Serial.print("OUTPUT =");  Serial.println(OUTPUT, DEC);
  //true|false
  blFlagT = true;
  blFlagF = false;
  //integer constants
  cData  = 128;       // integer decimal
  Serial.print("cData=");  Serial.println(cData, DEC);
  iData = 123;         // integer decimal
  Serial.print("123=");  Serial.println(iData);
  ucData  = B1111011;  // integer binary
  Serial.print("B1111011=");  Serial.println(ucData, DEC);
  iData = 0173;        // integer octal
  Serial.print("0173=");  Serial.println(iData);
  iData = 0x7B;        // integer hexadecimal
  Serial.print("0x7B=");  Serial.println(iData);
  uiData = 33u;        // integer unsigned
  Serial.print("33u=");  Serial.println(uiData);
  lData = 100000L;     // integer long
  Serial.print("100000L=");  Serial.println(lData);
  ulData = 32767ul;    // integer unsigned long
  Serial.print("32767ul=");  Serial.println(ulData);
  //floating point constants
  fData = 0.05;
  Serial.print("0.05=");  Serial.println(fData);
  fData = 10.0;
  fData = 2.34E5;
  Serial.print("2.34E5=");  Serial.println(fData);
  dData = 67e-12;
  Serial.print("67e-12=");  Serial.println(dData);

  /* Variable Scope & Qualifiers */
  Serial.println("# Variable Scope & Qualifiers");
  // variable scope
  Serial.println("## variable scope");
  int sameasglobal = 1;
  Serial.print("in setup() sameasglobal=");
  Serial.println(sameasglobal);
  globalcheck();
  // static
  Serial.println("## static & volatile");
  Serial.print("1st call : ");
  Serial.println(staticfunction());
  Serial.print("2nd call : ");
  Serial.println(staticfunction());
  /* Utilities */
  Serial.println("# Utilities ");
  //sizeof()
  Serial.println("## sizeof() ");
  Serial.print("char size is ");
  Serial.println(sizeof(char));
  Serial.print("short size is ");
  Serial.println(sizeof(short));  
  Serial.print("int size is ");
  Serial.println(sizeof(int));  
  Serial.print("long size is ");
  Serial.println(sizeof(long));  
  Serial.print("boolean size is ");
  Serial.println(sizeof(boolean));  
  Serial.print("byte size is ");
  Serial.println(sizeof(byte));  
  Serial.print("word size is ");
  Serial.println(sizeof(word));  
  Serial.print("float size is ");
  Serial.println(sizeof(float));  
  Serial.print("double size is ");
  Serial.println(sizeof(double));  
  Serial.print("strBuf[10] size is ");
  Serial.println(sizeof(strBuf));  

  //cast
  Serial.println("## cast ");
  Serial.println("cast char()");
  char cTarget;
  cTarget = char(blFlagT);
  Serial.println(cTarget, DEC);
  cTarget = char(cData);
  Serial.println(cTarget, DEC);
  cTarget = char(ucData);
  Serial.println(cTarget, DEC);
  cTarget = char(iData);
  Serial.println(cTarget, DEC);
  cTarget = char(lData);
  Serial.println(cTarget, DEC);
  cTarget = char(fData);
  Serial.println(cTarget, DEC);
  cTarget = char(dData);
  Serial.println(cTarget, DEC);

  Serial.println("cast byte()");
  byte bTarget;
  bTarget = byte(blFlagT);
  Serial.println(bTarget, DEC);
  bTarget = byte(cData);
  Serial.println(bTarget, DEC);
  bTarget = byte(ucData);
  Serial.println(bTarget, DEC);
  bTarget = byte(iData);
  Serial.println(bTarget, DEC);
  bTarget = byte(lData);
  Serial.println(bTarget, DEC);
  bTarget = byte(fData);
  Serial.println(bTarget, DEC);
  bTarget = byte(dData);
  Serial.println(bTarget, DEC);

  Serial.println("cast int()");
  int iTarget;
  iTarget = int(blFlagT);
  Serial.println(iTarget, DEC);
  iTarget = int(cData);
  Serial.println(iTarget, DEC);
  iTarget = int(ucData);
  Serial.println(iTarget, DEC);
  iTarget = int(iData);
  Serial.println(iTarget, DEC);
  iTarget = int(lData);
  Serial.println(iTarget, DEC);
  iTarget = int(fData);
  Serial.println(iTarget, DEC);
  iTarget = int(dData);
  Serial.println(iTarget, DEC);

  Serial.println("cast word()");
  word wTarget;
  wTarget = word(blFlagT);
  Serial.println(wTarget, DEC);
  wTarget = word(cData);
  Serial.println(wTarget, DEC);
  wTarget = word(ucData);
  Serial.println(wTarget, DEC);
  wTarget = word(iData);
  Serial.println(wTarget, DEC);
  wTarget = word(lData);
  Serial.println(wTarget, DEC);
  wTarget = word(fData);
  Serial.println(wTarget, DEC);
  wTarget = word(dData);
  Serial.println(wTarget, DEC);

  Serial.println("cast long()");
  long lTarget;
  lTarget = long(blFlagT);
  Serial.println(lTarget, DEC);
  lTarget = long(cData);
  Serial.println(lTarget, DEC);
  lTarget = long(ucData);
  Serial.println(lTarget, DEC);
  lTarget = long(iData);
  Serial.println(lTarget, DEC);
  lTarget = long(lData);
  Serial.println(lTarget, DEC);
  lTarget = long(fData);
  Serial.println(lTarget, DEC);
  lTarget = long(dData);
  Serial.println(lTarget, DEC);

  Serial.println("cast float()");
  float fTarget;
  fTarget = float(blFlagT);
  Serial.println(fTarget);
  fTarget = float(cData);
  Serial.println(fTarget);
  fTarget = float(ucData);
  Serial.println(fTarget);
  fTarget = float(iData);
  Serial.println(fTarget);
  fTarget = float(lData);
  Serial.println(fTarget);
  fTarget = float(fData);
  Serial.println(fTarget);
  fTarget = float(dData);
  Serial.println(fTarget);

  /* functions */
  Serial.println("# functions");
  Serial.println("## digital I/O");
  //Digital I/O
  int val;
  Serial.println("digitalWrite(PIN_DOUT, HIGH)");
  digitalWrite(PIN_DOUT, HIGH);
  Serial.print("digitalRead(PIN_DIN):");
  Serial.println(digitalRead(PIN_DIN));
  Serial.println("digitalWrite(PIN_DOUT, LOW)");
  digitalWrite(PIN_DOUT, LOW);
  Serial.print("digitalRead(PIN_DIN):");
  Serial.println(digitalRead(PIN_DIN));
  //PullUp
  digitalWrite(PIN_DIN2, HIGH);
  Serial.print("PullUp   digitalRead(PIN_DIN2):");
  Serial.println(digitalRead(PIN_DIN2));
  //PullDown
  digitalWrite(PIN_DIN2, LOW);
  Serial.print("PullDown digitalRead(PIN_DIN2):");
  Serial.println(digitalRead(PIN_DIN2));

  pinMode(A0, INPUT);
  Serial.print("digitalRead(A0) : ");
  Serial.println(digitalRead(A0));
//
  Serial.println("## analog I/O");
  //Analog I/O
  Serial.print("analogRead(A0) : ");
  Serial.println(analogRead(A0));
  Serial.print("analogRead(0) : ");
  Serial.println(analogRead(0));
  
  analogWrite(PIN_AOUT, 128);
  Serial.print("analogWrite(PIN_AOUT, 128) : ");
  Serial.println(analogRead(1));

  analogReference(DEFAULT);
//  analogReference(INTERNAL);
  analogReference(EXTERNAL);
//  analogReference(INTERNAL1V1);
//  analogReference(INTERNAL2C56);

  //Advanced I/O
  Serial.println("## Advanced I/O");
  Serial.println("### tone");
  tone(PIN_TONE, 440);
  tone(PIN_TONE, 440, 1);
  noTone(PIN_TONE);
  Serial.println("### shiftout ");
  pinMode(PIN_SOUT1, OUTPUT);
  pinMode(PIN_SOUT2, OUTPUT);
  shiftOut(PIN_SOUT1, PIN_SCLK, MSBFIRST, 0xAA);
  shiftOut(PIN_SOUT2, PIN_SCLK, LSBFIRST, 0xAA);
  Serial.println("### pluseIn ");
  unsigned long dur;
  pinMode(PIN_PIN, INPUT);
  dur = pulseIn(PIN_PIN, HIGH);
  dur = pulseIn(PIN_PIN, HIGH, 180000L);

  //Time
  Serial.println("## Time");
  dur =  millis();
  delay(1000);
  unsigned long dur2 =  millis();
  Serial.print("delay(1000) : ");
  Serial.print(dur2);
  Serial.print(",");
  Serial.print(dur);
  Serial.print(",");
  Serial.println(dur2 - dur);
  dur = micros();
  delayMicroseconds(16383);
  dur2 = micros();
  Serial.print("delayMicroseconds(16383) : ");
  Serial.print(dur2);
  Serial.print(",");
  Serial.print(dur);
  Serial.print(",");
  Serial.println(dur2 - dur);
  
  //Math
  Serial.println("## Math");
  int imin = min(1, 100);
  Serial.print("min(1, 100) : ");
  Serial.println(imin);
  float fmin = min(-0.5, -0.1);
  Serial.print("min(-0.5, -0.1) : ");
  Serial.println(fmin);
  int imax = max(1, 100);
  Serial.print("max(1, 100) : ");
  Serial.println(imax);
  float fmax = max(-0.5, -0.1);
  Serial.print("max(-0.5, -0.1 : ");
  Serial.println(fmax);
  int iabs  = abs(-5);
  Serial.print("abs(-5) : ");
  Serial.println(iabs);
  float fabs = abs(-3.1);
  Serial.print("abs(-3.1) : ");
  Serial.println(fabs);
  int iconst = constrain(1, 10, 50);
  float fconst = constrain(0.8, -0.1, 2.5);
  Serial.print("constrain(0.8, -0.1, 2.5) : ");
  Serial.println(fconst);
  val = map(600, 0, 1023, 0, 255);
  Serial.print("map(600, 0, 1023, 0, 255) : ");
  Serial.println(val);
  val = map(2000, 0, 1023, 0, 255);
  Serial.print("map(2000, 0, 1023, 0, 255) : ");
  Serial.println(val);  
  double dPow = pow(10, 2.1);
  Serial.print("pow(10, 2.1) : ");
  Serial.println(dPow);
  int isq = sq(100);
  Serial.print("sq(100) : ");
  Serial.println(isq);
  unsigned int uisq = sq(65535);
  Serial.print("sq(65535) : ");
  Serial.println(uisq);
  double dSqrt = sqrt(uisq);
  Serial.print("sqrt(uisq) : ");
  Serial.println(dSqrt);
  dSqrt = sqrt(2);
  Serial.print("sqrt(2) : ");
  Serial.println(dSqrt);
  dSqrt = sqrt(-2);
  Serial.print("sqrt(-2) : ");
  Serial.println(dSqrt);
  Serial.print("round(10.1) : ");
  Serial.println(round(10.1));
  Serial.print("round(10.49) : ");
  Serial.println(round(10.49));
  Serial.print("round(10.5) : ");
  Serial.println(round(10.5));
  Serial.print("round(10.9) : ");
  Serial.println(round(10.9));
  
  //Trigonometry
  Serial.println("## Trigonometry");
  double dSin = sin(radians(10));
  Serial.print("sin(radians(10)) : ");
  Serial.println(dSin);
  double dCos = cos(radians(10));
  Serial.print("cos(radians(10)) : ");
  Serial.println(dCos);
  double dTan = tan(radians(10));
  Serial.print("tan(radians(10)) : ");
  Serial.println(dTan);
  dTan = tan(radians(90));
  Serial.print("tan(radians(90)) : ");
  Serial.println(dTan);
  double deg = degrees(pi);
  Serial.print("degrees(pi) : ");
  Serial.println(deg);
  
  //Random Numbers
  Serial.println("## Random Numbers");
  randomSeed(analogRead(0));
  long randNumber = random(300);
  Serial.print("random(300) : ");
  Serial.println(randNumber);
  randNumber = random(10, 20);
  Serial.print("random(10, 20) : ");
  Serial.println(randNumber);
  
  //Bits and Bytes
  Serial.println("## Bits and Bytes");
  byte lbyte = lowByte(0x12345678);
  Serial.print("lowByte(0x12345678) : ");
  Serial.println(lbyte, HEX);
  byte hbyte = highByte(0x12345678);
  Serial.print("highByte(0x12345678) : ");
  Serial.println(hbyte, HEX);
  //bitRead()
  Serial.print("bitRead(B11010001, 7) : ");
  Serial.println(bitRead(0B11010001, 7));
  Serial.print("bitRead(B11010001, 6) : ");
  Serial.println(bitRead(0B11010001, 6));
  Serial.print("bitRead(B11010001, 5) : ");
  Serial.println(bitRead(0B11010001, 5));
  Serial.print("bitRead(B11010001, 4) : ");
  Serial.println(bitRead(0B11010001, 4));
  Serial.print("bitRead(B11010001, 3) : ");
  Serial.println(bitRead(0B11010001, 3));
  Serial.print("bitRead(B11010001, 2) : ");
  Serial.println(bitRead(0B11010001, 2));
  Serial.print("bitRead(B11010001, 1) : ");
  Serial.println(bitRead(0B11010001, 1));
  Serial.print("bitRead(B11010001, 0) : ");
  Serial.println(bitRead(0B11010001, 0));
  //bitWrite()/bitSet()/bitClear()
  byte bwr = 255;
  Serial.print("before bitWrite : ");
  Serial.println(bwr, BIN);
  //bit7
  bitWrite(bwr, 7, 0);
  Serial.print("after bitWrite(bwr, 7, 0) : ");
  Serial.println(bwr, BIN);
  bitSet(bwr, 7);
  Serial.print("after bitSet(bwr, 7) : ");
  Serial.println(bwr, BIN);
  bitClear(bwr, 7);
  Serial.print("after bitClear(bwr, 7) : ");
  Serial.println(bwr, BIN);
  bitWrite(bwr, 7, 1);
  Serial.print("after bitWrite(bwr, 7, 1) : ");
  Serial.println(bwr, BIN);
  //bit6
  bitWrite(bwr, 6, 0);
  Serial.print("after bitWrite(bwr, 6, 0) : ");
  Serial.println(bwr, BIN);
  bitSet(bwr, 6);
  Serial.print("after bitSet(bwr, 6) : ");
  Serial.println(bwr, BIN);
  bitClear(bwr, 6);
  Serial.print("after bitClear(bwr, 6) : ");
  Serial.println(bwr, BIN);
  bitWrite(bwr, 6, 1);
  Serial.print("after bitWrite(bwr, 6, 1) : ");
  Serial.println(bwr, BIN);
  //bit5
  bitWrite(bwr, 5, 0);
  Serial.print("after bitWrite(bwr, 5, 0) : ");
  Serial.println(bwr, BIN);
  bitSet(bwr, 5);
  Serial.print("after bitSet(bwr, 5) : ");
  Serial.println(bwr, BIN);
  bitClear(bwr, 5);
  Serial.print("after bitClear(bwr, 5) : ");
  Serial.println(bwr, BIN);
  bitWrite(bwr, 5, 1);
  Serial.print("after bitWrite(bwr, 5, 1) : ");
  Serial.println(bwr, BIN);
  //bit4
  bitWrite(bwr, 4, 0);
  Serial.print("after bitWrite(bwr, 4, 0) : ");
  Serial.println(bwr, BIN);
  bitSet(bwr, 4);
  Serial.print("after bitSet(bwr, 4) : ");
  Serial.println(bwr, BIN);
  bitClear(bwr, 4);
  Serial.print("after bitClear(bwr, 4) : ");
  Serial.println(bwr, BIN);
  bitWrite(bwr, 4, 1);
  Serial.print("after bitWrite(bwr, 4, 1) : ");
  Serial.println(bwr, BIN);
  //bit3
  bitWrite(bwr, 3, 0);
  Serial.print("after bitWrite(bwr, 3, 0) : ");
  Serial.println(bwr, BIN);
  bitSet(bwr, 3);
  Serial.print("after bitSet(bwr, 3) : ");
  Serial.println(bwr, BIN);
  bitClear(bwr, 3);
  Serial.print("after bitClear(bwr, 3) : ");
  Serial.println(bwr, BIN);
  bitWrite(bwr, 3, 1);
  Serial.print("after bitWrite(bwr, 3, 1) : ");
  Serial.println(bwr, BIN);
  //bit2
  bitWrite(bwr, 2, 0);
  Serial.print("after bitWrite(bwr, 2, 0) : ");
  Serial.println(bwr, BIN);
  bitSet(bwr, 2);
  Serial.print("after bitSet(bwr, 2) : ");
  Serial.println(bwr, BIN);
  bitClear(bwr, 2);
  Serial.print("after bitClear(bwr, 2) : ");
  Serial.println(bwr, BIN);
  bitWrite(bwr, 2, 1);
  Serial.print("after bitWrite(bwr, 2, 1) : ");
  Serial.println(bwr, BIN);
  //bit1
  bitWrite(bwr, 1, 0);
  Serial.print("after bitWrite(bwr, 1, 0) : ");
  Serial.println(bwr, BIN);
  bitSet(bwr, 1);
  Serial.print("after bitSet(bwr, 1) : ");
  Serial.println(bwr, BIN);
  bitClear(bwr, 1);
  Serial.print("after bitClear(bwr, 1) : ");
  Serial.println(bwr, BIN);
  bitWrite(bwr, 1, 1);
  Serial.print("after bitWrite(bwr, 1, 1) : ");
  Serial.println(bwr, BIN);
  //bit0
  bitWrite(bwr, 0, 0);
  Serial.print("after bitWrite(bwr, 0, 0) : ");
  Serial.println(bwr, BIN);
  bitSet(bwr, 0);
  Serial.print("after bitSet(bwr, 0) : ");
  Serial.println(bwr, BIN);
  bitClear(bwr, 0);
  Serial.print("after bitClear(bwr, 0) : ");
  Serial.println(bwr, BIN);
  bitWrite(bwr, 0, 1);
  Serial.print("after bitWrite(bwr, 0, 1) : ");
  Serial.println(bwr, BIN);
  //bit()
  Serial.print("bit(7) : ");
  Serial.println(bit(7), HEX);
  Serial.print("bit(6) : ");
  Serial.println(bit(6), HEX);
  Serial.print("bit(5) : ");
  Serial.println(bit(5), HEX);
  Serial.print("bit(4) : ");
  Serial.println(bit(4), HEX);
  Serial.print("bit(3) : ");
  Serial.println(bit(3), HEX);
  Serial.print("bit(2) : ");
  Serial.println(bit(2), HEX);
  Serial.print("bit(1) : ");
  Serial.println(bit(1), HEX);
  Serial.print("bit(0) : ");
  Serial.println(bit(0), HEX);
  
  //External Interrupts
  Serial.println("## External Interrupts");
  //Interrupts
  noInterrupts();
  attachInterrupt(0, blink, LOW);
  interrupts();
  noInterrupts();
  detachInterrupt(0);
  attachInterrupt(0, blink, CHANGE);
  interrupts();
  noInterrupts();
  detachInterrupt(0);
  attachInterrupt(0, blink, RISING);
  interrupts();
  noInterrupts();
  detachInterrupt(0);
  attachInterrupt(0, blink, FALLING);
  interrupts();
  noInterrupts();
  detachInterrupt(0);
  interrupts();
  
  
  //Serial
  Serial.println("## Serial");
  Serial.println("### Serial.print/println");
#ifdef RXDUINO_API
  Serial1.begin(9600, SCI_SCI0P2x);
#else
  Serial1.begin(9600);
#endif
  if(Serial1.available() > 0 )
     Serial1.read();
  Serial1.peek();
  Serial1.flush();
  Serial1.end();
  //print type
  Serial.print("78");
  Serial.print(78);
  Serial.print(1.23456, 0);
  Serial.println("78");
  Serial.println(78);
  Serial.println(1.23456, 0);
  Serial.print(78, BYTE);
  Serial.print(78, BIN);
  Serial.print(78, OCT);
  Serial.print(78, DEC);
  Serial.print(78, HEX);
  Serial.println(78, BYTE);
  Serial.println(78, BIN);
  Serial.println(78, OCT);
  Serial.println(78, DEC);
  Serial.println(78, HEX);

  cData = 0x61;
  ucData = 0x62;
  //boolean(true)
  Serial.print(blFlagT);
  //boolean(false)
  Serial.print(blFlagF);
  //char
  Serial.print(cData);
  //unsigned char
  Serial.print(ucData);
  //int
  Serial.print(iData);
  //unsigned int
  Serial.print(uiData);
  //long
  Serial.print(lData);
  //unsigned long 
  Serial.print(ulData);
  //float
  Serial.print(fData);
  //double
  Serial.print(dData);
  //char[]
  Serial.print(strBuf);
  //String
#ifdef RXDUINO_API
  Serial.print("");
#else
  Serial.print(stringStr);
#endif
  //boolean(true)
  Serial.println(blFlagT);
  //boolean(false)
  Serial.println(blFlagF);
  //char
  Serial.println(cData);
  //unsigned char
  Serial.println(ucData);
  //int
  Serial.println(iData);
  //unsigned int
  Serial.println(uiData);
  //long
  Serial.println(lData);
  //unsigned long 
  Serial.println(ulData);
  //float
  Serial.println(fData);
  //double
  Serial.println(dData);
  //char[]
  Serial.println(strBuf);
  //String
#ifdef RXDUINO_API
  Serial.print("");
#else
  Serial.println(stringStr);
#endif
  //void
  Serial.println();

  //negative
  Serial.println("### Serial.print negative char");
  char c = 128;
  Serial.println(c);
  Serial.println(c, BIN);
  Serial.println(c, OCT);
  Serial.println(c, DEC);
  Serial.println(c, HEX);

  //write  
  Serial.println("### Serial.write");
  Serial.write(65);
  Serial.write(ucData);
  Serial.write("BCD");
  Serial.write(strBuf);
  //unsigned char[]
  ucStr[0] = 'E';
  Serial.write(ucStr, 1);

  Serial.println("");
}

void loop()
{
}

